@@ -1,6 +1,6 @@
 /*
 babyphone - A baby monitor application on the Nokia N900.
-    Copyright (C) 2010  Roman Morawek <maemo@morawek.at>
+    Copyright (C) 2011  Roman Morawek <maemo@morawek.at>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -33,8 +33,7 @@ babyphone - A baby monitor application on the Nokia N900.
   As a last step it starts the audio monitoring.
 */
 MainWindow::MainWindow(QWidget *parent) :
-    QMainWindow(parent),
-    ui(new Ui::MainWindow)
+    QMainWindow(parent), ui(new Ui::MainWindow)
 {
     // load settings
     itsSettings = new Settings(this);
@@ -42,45 +41,38 @@ MainWindow::MainWindow(QWidget *parent) :
     // setup profile switcher
     itsProfileSwitcher = new ProfileSwitcher(itsSettings, this);
 
-    // setup audio monitor
-    itsAudioMonitor = new AudioMonitor(itsSettings, this);
-    connect(itsAudioMonitor, SIGNAL(update(int, int)), this, SLOT(refreshAudioData(int, int)));
-    itsMonitorActive = false;
-    itsNotificationPending = false;
-    itsIsScreenOff = false;
+    // setup UI, orientation and audio graphs
+    setupGui();
 
-    // setup GUI
-    ui->setupUi(this);
-    setOrientation(ScreenOrientationAuto);
-    // the horizontal layout is the pre-defined layout of the form classes
-    itsHorizontalLayout = true;
     itsActivationDelayTimer.setSingleShot(true);
-    connect(&itsActivationDelayTimer, SIGNAL(timeout()), this, SLOT(activationTimerExpired()));
+    connect(&itsActivationDelayTimer, SIGNAL(timeout()),
+            this, SLOT(activationTimerExpired()));
 
     // setup display status monitor
     bool result = QDBusConnection::systemBus().connect("",
                           "", "com.nokia.mce.signal", "display_status_ind",
                           this, SLOT(displayDimmed(const QDBusMessage&)));
     if (result == false)
-        qWarning() << tr("Cannot connect to display status: ") << QDBusConnection::systemBus().lastError();
-
-    // setup default UI values
-    ui->label_contact->setText(itsSettings->itsContact.GetDisplayString());
-
-    // setup graphs
-    itsAudioLevelGraphics = new AudioLevelGraphicsScene(ui->graphicsViewLevel, itsSettings, this);
-    connect(ui->graphicsViewLevel, SIGNAL(sizeChanged(QGraphicsView*)),
-            itsAudioLevelGraphics, SLOT(UpdateSize(QGraphicsView*)));
-    itsAudioCounterGraphics = new AudioLevelGraphicsScene(ui->graphicsViewCounter, itsSettings, this);
-    connect(ui->graphicsViewCounter, SIGNAL(sizeChanged(QGraphicsView*)),
-            itsAudioCounterGraphics, SLOT(UpdateSize(QGraphicsView*)));
+        qWarning() << "Cannot connect to display status: " << QDBusConnection::systemBus().lastError();
 
     // show information message at first startup
     if (itsSettings->itsFirstRun)
         showFirstRunInfo();
 
-    // start audio capturing
-    startAudio();
+    // start babyphone engine
+    itsBabyphone = new Babyphone(itsSettings, this);
+    // register for audio data to update display
+    connect(itsBabyphone, SIGNAL(newAudioData(int,int)),
+            this, SLOT(newAudioData(int,int)));
+    // register to phone call info
+    connect(itsBabyphone, SIGNAL(newCallStatus(bool,bool)),
+            this, SLOT(newCallStatus(bool,bool)));
+    // register for end of phone application to put babyphone to forground again
+    connect(itsBabyphone, SIGNAL(phoneApplicationFinished()),
+            this, SLOT(bringWindowToFront()));
+    // display potential errors
+    connect(itsBabyphone, SIGNAL(notificationError()),
+            this, SLOT(showNotificationError()));
 }
 
 
@@ -97,6 +89,34 @@ MainWindow::~MainWindow()
 }
 
 
+void MainWindow::setupGui()
+{
+    // setup GUI
+    ui->setupUi(this);
+
+    // set rotation lock
+    if (itsSettings->itsDisableAutoRotate)
+        setOrientation(ScreenOrientationLockLandscape);
+    else
+        setOrientation(ScreenOrientationAuto);
+    // the horizontal layout is the pre-defined layout of the form classes
+    // we need to set this, even if we start in portrait mode
+    itsHorizontalLayout = true;
+    itsIsScreenOff = false;
+
+    // setup default UI values
+    ui->label_contact->setText(itsSettings->itsContact.GetDisplayString());
+
+    // setup graphs
+    itsAudioLevelGraphics = new AudioLevelGraphicsScene(ui->graphicsViewLevel, itsSettings, this);
+    connect(ui->graphicsViewLevel, SIGNAL(sizeChanged(QGraphicsView*)),
+            itsAudioLevelGraphics, SLOT(UpdateSize(QGraphicsView*)));
+    itsAudioCounterGraphics = new AudioLevelGraphicsScene(ui->graphicsViewCounter, itsSettings, this);
+    connect(ui->graphicsViewCounter, SIGNAL(sizeChanged(QGraphicsView*)),
+            itsAudioCounterGraphics, SLOT(UpdateSize(QGraphicsView*)));
+}
+
+
 /*!
   resizeEvent takes care for layout switches (horizonatal/vertical).
 */
@@ -179,6 +199,12 @@ void MainWindow::on_actionSettings_triggered()
     if (dialog.exec() == QDialog::Accepted)
     {
         dialog.SaveData();
+
+        // update orientation lock, even if unchanged
+        if (itsSettings->itsDisableAutoRotate)
+            setOrientation(ScreenOrientationLockLandscape);
+        else
+            setOrientation(ScreenOrientationAuto);
     }
 }
 
@@ -247,7 +273,7 @@ void MainWindow::on_pushButton_clicked()
     itsActivationDelayTimer.stop();
 
     // did we switch ON or OFF?
-    if (itsMonitorActive) {
+    if (ui->pushButton->isChecked() == false) {
         // switch OFF
         deactivateMonitor();
     }
@@ -291,16 +317,11 @@ void MainWindow::activateMonitor()
         itsAudioCounterGraphics->Clear();
     }
 
-    // setup phone call handlers
-    itsCallMonitor = new CallMonitor(itsSettings, this);
-    itsUserNotifier = new UserNotifier(itsSettings, itsCallMonitor);
-    connect(itsUserNotifier, SIGNAL(notifyFinished()), this, SLOT(notifyFinished()));
-
     // start activation timeout
     itsActivationDelayTimer.start(itsSettings->itsActivationDelay*1000);
 
-    // the active flag is set immediately
-    itsMonitorActive = true;
+    // inform state machine on new state
+    itsBabyphone->setState(Babyphone::STATE_INACTIVE_ON);
 }
 
 
@@ -312,33 +333,15 @@ void MainWindow::activateMonitor()
 void MainWindow::deactivateMonitor()
 {
     // switch OFF
-    itsMonitorActive = false;
+    itsBabyphone->setState(Babyphone::STATE_OFF);
 
     // if demanded, show call statistics
     if (itsSettings->itsShowStatistics) {
-        QString text;
-        // setup notification text
-        if (itsUserNotifier->itsCallCounterInvoke > 0) {
-            text = tr("Notifications total: %1\n"
-                      "Calls taken: %2\n"
-                      "Unanswered or timed out: %3")
-                    .arg(itsUserNotifier->itsCallCounterInvoke)
-                    .arg(itsUserNotifier->itsCallCounterTaken)
-                    .arg(itsUserNotifier->itsCallCounterTimeout);
-        }
-        else {
-            text = tr("No notifications took place.");
-        }
-
         // show statistics
-        QMessageBox::information(this, tr("Call Notification Statistics"), text);
+        QString statistics = itsBabyphone->getStatistics();
+        QMessageBox::information(this, tr("Call Notification Statistics"), statistics);
     }
 
-    // terminate phone call handler
-    disconnect(itsUserNotifier, SIGNAL(notifyFinished()), this, SLOT(notifyFinished()));
-    delete itsUserNotifier;
-    delete itsCallMonitor;
-
     // update GUI
     ui->pushButton->setText(tr("Inactive - Click to start"));
     ui->pushButton_contacts->setEnabled(true);
@@ -351,6 +354,21 @@ void MainWindow::deactivateMonitor()
 }
 
 
+void MainWindow::newCallStatus(bool finish, bool selfInitiated)
+{
+    ui->pushButton->setEnabled(finish);
+
+    // check for reactivation delay
+    if (finish && selfInitiated) {
+        itsBabyphone->setState(Babyphone::STATE_INACTIVE_ON);
+
+        // start activation timeout
+        ui->pushButton->setText(tr("Active - Waiting for reactivation delay"));
+        itsActivationDelayTimer.start(itsSettings->itsRecallTimer*1000);
+    }
+}
+
+
 /*!
   activationTimerExpired gets called as the activation actually starts.
   It just modifies the button's label.
@@ -358,36 +376,7 @@ void MainWindow::deactivateMonitor()
 void MainWindow::activationTimerExpired()
 {
     ui->pushButton->setText(tr("Active - Click to Stop"));
-}
-
-
-/*!
-  notifyFinished represents the slot that gets called as the UserNotifier
-  finished its phone call and switches the monitoring state to active again.
-  This is done by starting a single shot timer with the configured recall
-  timeout to inhibit immediate re-calling of the parents.
-*/
-void MainWindow::notifyFinished()
-{
-    // notifcation finished
-    itsNotificationPending = false;
-
-    // update GUI
-    ui->pushButton->setEnabled(true);
-    ui->pushButton->setText(tr("Active - Waiting for reactivation delay"));
-
-    // bring the application back to focus
-    // we need a timeout for the phone application to quit
-    QTimer::singleShot(itsSettings->REFOCUS_TIMER, this, SLOT(bringWindowToFront()));
-
-    // reset audio monitor warning
-    itsAudioMonitor->itsCounter = 0;
-
-    // start activation timeout
-    itsActivationDelayTimer.start(itsSettings->itsRecallTimer*1000);
-
-    // restart audio monitoring
-    startAudio();
+    itsBabyphone->setState(Babyphone::STATE_ACTIVE_ON);
 }
 
 
@@ -403,75 +392,15 @@ void MainWindow::bringWindowToFront()
 
 
 /*!
-  startAudio performs the periodical reactivation of the audio capturing.
-  There is a repeating activate and deactivate with given timeouts to keep the
-  audio stream synchronous to real time. This needs to be considered as a
-  work around to inhibit audio capturing problems.
+  newAudioData updates the audio graphs.
 */
-void MainWindow::startAudio()
-{
-    itsAudioMonitor->start();
-
-    // setup safety timer
-    QTimer::singleShot(itsSettings->AUDIO_ON_TIMER, this, SLOT(stopAudio()));
-}
-
-
-/*!
-  stopAudio performs the periodical deactivation of the audio capturing.
-  See startAudio for further explaination.
-*/
-void MainWindow::stopAudio()
-{
-    // suspend audio monitoring
-    itsAudioMonitor->stop();
-
-    // in case of notification, we get an event on its end
-    // otherwise we must restart automatically via a timer
-    if (!itsNotificationPending)
-        QTimer::singleShot(itsSettings->AUDIO_OFF_TIMER, this, SLOT(startAudio()));
-}
-
-
-/*!
-  refreshAudioData periodically receives the audio samples from the AudioMonitor,
-  updates the audio graphs and performs the threshold check to initiate a phone
-  call if needed. If the phone call initiation failed it displays an error
-  pop-up.
-*/
-void MainWindow::refreshAudioData(int counter, int value)
+void MainWindow::newAudioData(int counter, int value)
 {
     // update GUI if inactive or if set to always update
     if (!itsIsScreenOff) {
         itsAudioLevelGraphics->AddValue(value);
         itsAudioCounterGraphics->AddValue(counter);
     }
-
-    // check for noise
-    if (itsMonitorActive && !itsActivationDelayTimer.isActive()
-        && (counter > itsSettings->THRESHOLD_VALUE))
-    {
-        qDebug() << tr("Audio threshold reached. Notifying user.");
-
-        // store event
-        itsNotificationPending = true;
-
-        // update GUI
-        ui->pushButton->setEnabled(false);
-
-        // stop monitoring
-        stopAudio();
-
-        // notify user
-        if (itsUserNotifier->Notify() == false)
-        {
-            // the notify command yielded an error
-            showNotificationError();
-
-            // restart audio capturing
-            notifyFinished();
-        }
-    }
 }
 
 
@@ -550,6 +479,6 @@ void MainWindow::showFirstRunInfo() const
     // display message
     QDBusMessage reply = QDBusConnection::systemBus().call(msg);
     if (reply.type() == QDBusMessage::ErrorMessage) {
-        qWarning() << tr("Could not display information message.");
+        qWarning() << "Could not display information message.";
     }
 }
